@@ -36,16 +36,16 @@ sub new {
 }
 
 sub find {
-    my ( $this, $cep ) = @_;
+    my ( $this, $cep, $as_html_tree ) = @_;
 
-    my @list_address = $this->_extractAddress($cep);
+    my @list_address = $this->_extractAddress( $cep, $as_html_tree );
     $list_address[0]{address_count} = @list_address unless wantarray;
 
     return wantarray ? @list_address : $list_address[0];
 }
 
 sub _extractAddress {
-    my ( $this, $cep ) = @_;
+    my ( $this, $cep, $as_html_tree ) = @_;
 
     my @result = ();
 
@@ -78,7 +78,7 @@ sub _extractAddress {
 
             # Check the outcome of the response
             if ( $res->is_success ) {
-                $this->_parseHTML( \@result, $res->content );
+                $this->_parseHTML( \@result, $res->content, $as_html_tree );
             }
             else {
                 $result[0]->{status} = "Error: " . $res->status_line;
@@ -92,7 +92,7 @@ sub _extractAddress {
 }
 
 sub _parseHTML {
-    my ( $this, $address_ref, $html ) = @_;
+    my ( $this, $address_ref, $html, $as_html_tree ) = @_;
 
     my $tree = HTML::TreeBuilder::XPath->new;
     $html = decode( 'iso-8859-1', $html );
@@ -103,24 +103,30 @@ sub _parseHTML {
       $tree->findnodes('//table[contains(@class,"tmptabela")]/tr[not(th)]');
 
     while ( my $p = shift(@$ref) ) {
-        my $address = {};
 
-        $address->{street}       = $p->findvalue('./td[1]');
-        $address->{neighborhood} = $p->findvalue('./td[2]');
-        $address->{cep}          = $p->findvalue('./td[4]');
-
-        ( $address->{location}, $address->{uf} ) =
-          split qr{\s*/\s*} => $p->findvalue('./td[3]');
-
-        if ( $address->{cep} ) {
-            $address->{status} = '';
+        if ($as_html_tree) {
+            push( @$address_ref, $p );
         }
         else {
-            $address->{status} =
-              'Error: Address not found, something is wrong...';
-        }
+            my $address = {};
 
-        push( @$address_ref, $address );
+            $address->{street}       = $p->findvalue('./td[1]');
+            $address->{neighborhood} = $p->findvalue('./td[2]');
+            $address->{cep}          = $p->findvalue('./td[4]');
+
+            ( $address->{location}, $address->{uf} ) =
+              split qr{\s*/\s*} => $p->findvalue('./td[3]');
+
+            if ( $address->{cep} ) {
+                $address->{status} = '';
+            }
+            else {
+                $address->{status} =
+                  'Error: Address not found, something is wrong...';
+            }
+
+            push( @$address_ref, $address );
+        }
     }
 
     $address_ref->[0]->{status} = 'Error: Address not found'
@@ -142,9 +148,9 @@ WWW::Correios::CEP - Perl extension for extract address from CEP (zip code) numb
 
     use WWW::Correios::CEP;
 
-    my $cepper = WWW::Correios::CEP-new;
+    my $cep = WWW::Correios::CEP->new();
 
-    my $address = $cepper->find( $cep );
+    my $address = $cep->find( $cep );
 
     print $address->{street}; # neighborhood, location, uf
 
@@ -153,6 +159,12 @@ WWW::Correios::CEP - Perl extension for extract address from CEP (zip code) numb
 This module fetches CEP information (Brazilian ZIP codes) directly from
 the Correios website, Brazil's official post office company.
 
+=head2 Good to know
+
+Also, check if the returned CEP matches the informed CEP, some addresses can changed from time to time, for example, 49039-050 is now 49009-010 (today is 2020-07-04).
+Correios still return the address street name, but some providers may not accept it anymore (eg: payment gateways)
+
+Correios API is sometimes unstable, please have a fallback!
 
 =head1 METHODS
 
@@ -184,7 +196,7 @@ Extra options to pass to LWP::UserAgent.
 
 =back
 
-=head2 find( $cep )
+=head2 find( $cep [, $as_html_tree ] )
 
 Recieves the CEP string and tries to get address data. Returns a hashref with the following keys:
 
@@ -206,6 +218,8 @@ If there is more than one address, it returns a list of hashrefs in list context
 just the first hashref in scalar context, together with an "C<address_count>" key with
 the total returned addresses.
 
+If $as_html_tree is passed, the list hahsref is changed to raw object of HTML::TreeBuilder::XPath->findnodes()
+
 =head1 SEE ALSO
 
 WWW::Correios::SRO
